part of jaguar.generator.parser.route;

class ParsedRoute {
  final ParsedUpper upper;

  final MethodElementWrap method;

  final AnnotationElementWrap routeAnnot;

  final List<Symbol> interceptors;

  final List<ParsedExceptionHandler> exceptions;

  ant.RouteBase get item => routeAnnot.instantiated;

  String get instantiationString => routeAnnot.instantiationString;

  String get prototype => method.prototype;

  bool get returnsResponse =>
      method.returnTypeWithoutFuture.compareNamedElement(kJaguarResponse);

  DartTypeWrap get jaguarResponseType {
    if (!returnsResponse) {
      return method.returnTypeWithoutFuture;
    }

    return method.returnTypeWithoutFuture.typeArguments.first;
  }

  ParsedRoute(this.upper, this.method, this.routeAnnot, this.interceptors,
      this.exceptions);
}

class _ParsedRouteBuilder {
  final ParsedUpper upper;

  final MethodElementWrap method;

  ParsedRoute _route;

  ParsedRoute get route => _route;

  _ParsedRouteBuilder(this.upper, this.method) {
    List<AnnotationElementWrap> annots = method.metadata
        .where((annot) => annot.instantiated is ant.RouteBase)
        .toList();

    if (annots.length == 0) {
      return;
    }

    if (annots.length != 1) {
      final except = new RouteException(
          'Route method has more than one route annotations!');
      except.upper = upper.name;
      except.route = method.name;
      throw except;
    }

    final List<Symbol> interceptors = detectWrappers(upper.upper, method);

    final List<ParsedExceptionHandler> exceptions = [];
    ParsedExceptionHandler.detectAllExceptions(method).forEach(exceptions.add);

    _route =
        new ParsedRoute(upper, method, annots.first, interceptors, exceptions);
  }
}
