library jaguar_generator.writer;

import 'dart:mirrors';
import 'package:jaguar_generator/models/models.dart';

part 'route_call.dart';
part 'exception.dart';

class Writer {
  Upper _u;

  Writer(this._u);

  List<Route> get _r => _u.routes;

  List<Group> get _g => _u.groups;

  String generate() {
    StringBuffer sb = new StringBuffer();
    sb.writeln("class ${_u.name} implements RequestHandler {");

    sb.writeln(_writeRouteList());

    sb.writeln("final ${_u.baseName} _internal;");

    sb.writeln(_writeGroupDecl());

    sb.writeln(_writeConstructors());

    sb.writeln(_writeReqHandler());

    sb.writeln('}');

    return sb.toString();
  }

  String _writeConstructors() {
    StringBuffer sb = new StringBuffer();

    sb.write('${_u.name}(this._internal)');

    if (_u.groups.length != 0) {
      sb.write(':');
      sb.write(_u.groups
          .map((Group g) =>
              '_${g.name}Internal = new Jaguar${g.type}(_internal.${g.name})')
          .join(','));
    }

    sb.writeln(';');

    return sb.toString();
  }

  String _writeRouteList() {
    StringBuffer sb = new StringBuffer();

    sb.writeln("static const List<RouteBase> routes = const <RouteBase>[");
    String routeList = _r.map((Route route) => route.instantiation).join(',');
    sb.write(routeList);
    sb.writeln("];");

    return sb.toString();
  }

  String _writeGroupDecl() {
    StringBuffer sb = new StringBuffer();

    _g.forEach((Group group) {
      sb.writeln('final Jaguar${group.type} _${group.name}Internal;');
    });

    return sb.toString();
  }

  String _writeReqHandler() {
    StringBuffer sb = new StringBuffer();
    sb.writeln(kHandleReq);
    if (_u.prefix.isNotEmpty) {
      sb.writeln("prefix += '${_u.prefix}';");
    }

    if (_r.isNotEmpty) {
      sb.writeln("final ctx = new Context(request);");
      sb.writeln("bool match = false;");
    }

    sb.writeln();

    for (int i = 0; i < _r.length; i++) {
      sb.writeln(_writeHandlerForRoute(_r[i], i));
    }

    _u.groups.forEach((Group group) {
      sb.writeln("{");
      sb.write("Response response = ");
      sb.write(
          "await _${group.name}Internal.handleRequest(request,prefix: prefix");
      if (group.path.isNotEmpty) {
        sb.write(" + '${group.path}'");
      }
      sb.writeln(");");

      sb.writeln("if (response is Response) {");
      sb.writeln("return response;");
      sb.writeln("}");
      sb.writeln("}");
      sb.writeln();
    });

    sb.writeln("return null;");
    sb.writeln("}");
    sb.writeln();
    return sb.toString();
  }

  String _writeHandlerForRoute(final Route route, final int i) {
    StringBuffer sb = new StringBuffer();
    sb.writeln('//Handler for ${route.method.name}');
    sb.writeln(
        "match = routes[$i].match(request.uri.path, request.method, prefix, ctx.pathParams);");
    sb.writeln("if (match) {");

    if (route.exceptions.length != 0) {
      sb.writeln("try {");
    }

    sb.writeln("final interceptors = <Interceptor>[];");

    // create interceptors
    for(Symbol interceptor in route.interceptors) {
      final String name = MirrorSystem.getName(interceptor);
      sb.writeln("interceptors.add(_internal.${name}(ctx));");
    }

    {
      RouteCallWriter writer = new RouteCallWriter(route, i);
      sb.write(writer.generate());
    }

    if (route.exceptions.length != 0) {
      sb.write('} ');

      RouteExceptionWriter exceptWriter = new RouteExceptionWriter(route);
      sb.write(exceptWriter.generate());
    }

    sb.writeln("}");
    sb.writeln();

    return sb.toString();
  }
}

const String kHandleReq =
    "Future<Response> handleRequest(Request request, {String prefix: ''}) async {";
